REM     Convert the MATPAR output data file SPAR.DAT
REM     to a Touchstone-compatible file (for n=1, 2, 3, and 4 ports),
REM     which has the default extension

	pi = 4 * ATN(1)
	rdcf = 180 / pi

REM     Determine the number of ports (n)

	ON ERROR GOTO 1
	OPEN "spar.dat" FOR INPUT AS #1
	GOTO 2
1       PRINT "Cannot open file SPAR.DAT"
	GOTO 10

2       ON ERROR GOTO 3
	INPUT #1, line$
	INPUT #1, line$
	FOR i = 1 TO 11
		INPUT #1, line$
		IF line$ = "" GOTO 4
	NEXT i

	PRINT "The number of ports exceeds 4"
	GOTO 10
3       PRINT "Error reading file SPAR.DAT"
	GOTO 10

4       IF i = 2 THEN
		n = 1
	ELSEIF i = 4 THEN
		n = 2
	ELSEIF i = 7 THEN
		n = 3
	ELSEIF i = 11 THEN
		n = 4
	ELSE
		PRINT "Error parsing file SPAR.DAT"
		GOTO 10
	END IF

	CLOSE #1

REM     Start conversion

	ON ERROR GOTO 1
	OPEN "spar.dat" FOR INPUT AS #1
	ON ERROR GOTO 5

REM     Get Touchstone-compatible file name

	INPUT "Touchstone file name (without extension) ", line$
	IF n = 1 THEN
		line$ = line$ + ".s1p"
	ELSEIF n = 2 THEN
		line$ = line$ + ".s2p"
	ELSEIF n = 3 THEN
		line$ = line$ + ".s3p"
	ELSE
		line$ = line$ + ".s4p"
	END IF
       
REM     Open Touchstone-compatible file name and write header     

	OPEN line$ FOR OUTPUT AS #2
	IF n = 1 THEN
		PRINT #2, "! Scattering parameters of a network with"; 1; "port"
	ELSE
	  PRINT #2, "! Scattering parameters of a network with"; n; "ports"
	END IF
	GOTO 6
5       PRINT "Cannot open file "; line$
	GOTO 10

6       ON ERROR GOTO 8
	DIM sm(1 TO n, 1 TO n), sa(1 TO n, 1 TO n)
     
REM     define formats for frequencies (in GHz) and scattering parameters
REM     (magnitude and angle in degrees)

	fmtf$ = "##.####^^^^ "
	fmts$ = "#.#### ####.## "
	fmtfs1$ = fmtf$ + fmts$
	fmts3$ = fmts$ + fmts$ + fmts$
	fmtfs3$ = fmtf$ + fmts3$
	fmts4$ = fmts3$ + fmts$
	fmtfs4$ = fmtf$ + fmts4$
	fmtfs2$ = fmtfs4$
	tabf = LEN(fmtf$) + 1

REM     Cycle for each frequency

REM     Read frequency in Hz and convert it to GHz

7       INPUT #1, f
	f = f / 1E+09
	INPUT #1, line$
       
REM     Read and convert the lower triangle and the main diagonal
REM     of the scattering matrix

	FOR j = 1 TO n
		FOR k = 1 TO j
		      
REM     Read a scattering parameter in rectangular form
REM     and convert it to polar form

			INPUT #1, sr, si
			sm(j, k) = SQR(sr ^ 2 + si ^ 2)
			IF sm(j, k) = 0 THEN
				sa(j, k) = 0
			ELSEIF sr > 0 THEN
				sa(j, k) = ATN(si / sr) * rdcf
			ELSEIF sr = 0 AND si >= 0 THEN
				sa(j, k) = 90
			ELSEIF sr = 0 AND si < 0 THEN
				sa(j, k) = -90
			ELSEIF sr < 0 AND si >= 0 THEN
				sa(j, k) = ATN(si / sr) * rdcf + 180
			ELSE
				sa(j, k) = ATN(si / sr) * rdcf - 180
			END IF

REM     Fill the upper triangle of the scattering matrix

			IF k <> j THEN
				sm(k, j) = sm(j, k)
				sa(k, j) = sa(j, k)
			END IF
		NEXT k
	NEXT j

REM     Print the converted data

	IF n = 1 THEN
		PRINT #2, USING fmtfs1$; f; sm(1, 1); sa(1, 1)
	ELSEIF n = 2 THEN
		PRINT #2, USING fmtfs2$; f; sm(1, 1); sa(1, 1); sm(2, 1); sa(2, 1); sm(1, 2); sa(1, 2); sm(2, 2); sa(2, 2)
	ELSEIF n = 3 THEN
		PRINT #2, USING fmtfs3$; f; sm(1, 1); sa(1, 1); sm(1, 2); sa(1, 2); sm(1, 3); sa(1, 3)
		PRINT #2, USING fmts3$; TAB(tabf); sm(2, 1); sa(2, 1); sm(2, 2); sa(2, 2); sm(2, 3); sa(2, 3)
		PRINT #2, USING fmts3$; TAB(tabf); sm(3, 1); sa(3, 1); sm(3, 2); sa(3, 2); sm(3, 3); sa(3, 3)
	ELSE
		PRINT #2, USING fmtfs4$; f; sm(1, 1); sa(1, 1); sm(1, 2); sa(1, 2); sm(1, 3); sa(1, 3); sm(1, 4); sa(1, 4)
		PRINT #2, USING fmts4$; TAB(tabf); sm(2, 1); sa(2, 1); sm(2, 2); sa(2, 2); sm(2, 3); sa(2, 3); sm(2, 4); sa(2, 4)
		PRINT #2, USING fmts4$; TAB(tabf); sm(3, 1); sa(3, 1); sm(3, 2); sa(3, 2); sm(3, 3); sa(3, 3); sm(3, 4); sa(3, 4)
		PRINT #2, USING fmts4$; TAB(tabf); sm(4, 1); sa(4, 1); sm(4, 2); sa(4, 2); sm(4, 3); sa(4, 3); sm(4, 4); sa(4, 4)
	END IF
	INPUT #1, line$
	GOTO 7

REM     Close files

8       CLOSE #1
	CLOSE #2

10      PRINT "Job terminated"
       
	END

